// *spark audio-visual
// RS232 Control for TV-One products
// Good for 1T-C2-750, others will need some extra work
// Copyright *spark audio-visual 2009-2010

#include "WProgram.h"
#include "spk_tvone.h"

TVOne::TVOne()
{
  Serial.begin(57600);
}

boolean TVOne::command(uint8_t channel, uint8_t window, int32_t func, int32_t payload) 
{

  uint8_t cmd[8];
  uint8_t ret[8];
  uint8_t checksum = 0;
  boolean success = false;
  char i,j;
  
  // TASK: discard anything waiting to be read
  while (Serial.available() > 0) {
    Serial.read();
  }
  
  // TASK: Create command string, expressed as 8 bytes in ASCII

  // CMD
  cmd[0] = 1<<2; // write
  // CHA
  cmd[1] = channel;
  // WINDOW
  cmd[2] = window;
  // OUTPUT & FUNCTION
  //            cmd[3]  cmd[4]
  // output 0 = 0000xxx xxxxxxx
  // function = xxxXXXX XXXXXXX
  cmd[3] = func >> 8;
  cmd[4] = func & 0xFF;
  // PAYLOAD
  cmd[5] = (payload >> 16) & 0xFF;
  cmd[6] = (payload >> 8) & 0xFF;
  cmd[7] = payload & 0xFF;

  // TASK: Write command to RS232, wrapping and padding as appropriate

  for (i=0; i<8; i++) 
  {
    checksum += cmd[i];
  }
  
  char serialCommand[20];
  
  sprintf (serialCommand, "F%02X%02X%02X%02X%02X%02X%02X%02X%02X\r", cmd[0], cmd[1], cmd[2], cmd[3], cmd[4], cmd[5], cmd[6], cmd[7], checksum);
  
  Serial.print(serialCommand);
  
  // TASK: Check the unit's return string, to enable return to main program as soon as unit is ready
  unsigned long safeTime = millis() + 100;
  unsigned long clearTime = millis() + 30;
  while (millis() < safeTime) {
    // Handling the timing of this return is critical to effective control. Too short, the processor ignores commands; too long, the control is 'steppy'
    // According to the manual, operations typically take 30ms, and to simplify programming you can throttle commands to every 100ms
    // 100ms is too slow for us. Going with returning after 30ms if we've received an acknowledgement (strictly speaking, anything).
    // Returning the instant something is received back overloads the processor.
    // FIXME is to set up software serial and debug the return message parser of earlier revs
    if (Serial.available() >= 18) success = true;
    if (success && (clearTime < millis())) break;
  }

  return success;
}

void TVOne::setCustomResolutions() 
{
  set1920x480(k1920x480);
  set1600x600(k1600x600);
}

void TVOne::setHDCPOff() 
{
  // Turn HDCP off on the output
  command(0, kA, kOutputHDCPRequired, 0);
  command(0, kA, kOutputHDCPStatus, 0);
  // Likewise on inputs A and B
  command(0, kA, kHDCPAdvertise, 0);
  command(0, kA, kHDCPAdvertise, 0);
  command(0, kB, kHDCPStatus, 0);
  command(0, kB, kHDCPStatus, 0);
}

void TVOne::set1920x480(int resStoreNumber) 
{
  command(0, 0, kResImageToAdjust, resStoreNumber);
  command(0, 0, kResInterlaced, 0);
  command(0, 0, kResHFreqCoarse, 31400);
  command(0, 0, kResHFreqFine, 31475);
  command(0, 0, kHVActiveH, 1920);
  command(0, 0, kResHVActiveV, 480);
  command(0, 0, kResHVStartH, 192); 
  command(0, 0, kResHVStartV, 32); 
  command(0, 0, kResPixelsPerLine, 2400); 
  command(0, 0, kResLinesPerField, 525);
  command(0, 0, kResHVSyncH, 240);
  command(0, 0, kResHVSyncV, 5); 
  command(0, 0, kResSyncPolarity, 0);
}

void TVOne::set1600x600(int resStoreNumber) 
{
  command(0, 0, kResImageToAdjust, resStoreNumber);
  command(0, 0, kResInterlaced, 0);
  command(0, 0, kResHFreqCoarse, 37879);
  command(0, 0, kResHFreqFine, 37879);
  command(0, 0, kHVActiveH, 1600);
  command(0, 0, kResHVActiveV, 600);
  command(0, 0, kResHVStartH, 160); 
  command(0, 0, kResHVStartV, 1); 
  command(0, 0, kResPixelsPerLine, 2112); 
  command(0, 0, kResLinesPerField, 628);
  command(0, 0, kResHVSyncH, 192);
  command(0, 0, kResHVSyncV, 14); 
  command(0, 0, kResSyncPolarity, 0);
}
