// *spark audio-visual
// RS232 Control for TV-One products
// Good for 1T-C2-750, others will need some extra work
// Copyright *spark audio-visual 2009-2010

#ifndef SPK-TVOne_h
#define SPK-TVOne_h

// Arduino include
#include "WProgram.h"

// Res #27 640x480, 59.97Hz
// Res #28 640x480, 60Hz
// Res #99 1920x1080, 60Hz
// Res #104 = 1920x1200, 60Hz
// Res #112 = we appropriate for 1920x480, 60Hz
// Res #113 = we appropriate for 1600x600, 60Hz
#define k1920x480 112
#define k1600x600 113

// sources
#define kRGB1 0x10
#define kRGB2 0x11

// windows
#define kA 'A'
#define kB 'B'
#define kALogo 'a'
#define kBLogo 'b'
#define kZ 'Z'

// source commands (need to specify CHA)
#define kEDIDToUse 0x243 // 0..7 is 1-6, default, Mon
#define kHDCPAdvertise 0x237 // 0=Off, 1=On
#define kHDCPStatus 0x238 // 0=Inactive, 1=Active

// window commands
#define kWindowEnable 0x12B // 0 = Off, 1 = On
#define kMaxFadeLevel 0x10F // 0-100 is fade %
#define kFadeOutIn  0x193 // -1 fade out, 0 no action, 1 fade in

// output commands 
// combinations are: Switcher Output 1 (0x00) and Window A (0x41) / Z (0x5A) / Logo a (0x61); ie. pair with kA.
#define kOutputLockMethod 0x149 // 0 = Off 1 = Genlock 2 = Lock & Mix
#define kOutputRes 0x083 // 0...1000, presets in the firmware
#define kOutputHDCPRequired 0x233 // 0..1, Off, On (if display supports it)
#define kOutputHDCPStatus 0x234 // 0=Unavailable 1=Supported 2=Active 3=Repeater supported 4=Repeater active

// resolution commands
#define kResImageToAdjust 0x081 // 1..1000 
#define kResInterlaced 0x0CA // 0..1 = Off, On 
#define kResHFreqCoarse 0x0BE // 10000..200000 
#define kResHFreqFine 0x0BF // 10000..200000 
#define kHVActiveH 0x096 // 64..2047 
#define kResHVActiveV 0x097 // 64..2047 
#define kResHVStartH 0x08B // 0..1023 
#define kResHVStartV 0x08C // 0..1023 
#define kResPixelsPerLine 0x08D // aka Clks/l 64..4095 
#define kResLinesPerField 0x08E // 64..2047 
#define kResHVSyncH 0x08F //8..1023 
#define kResHVSyncV 0x090 // 1..1023 
#define kResSyncPolarity 0x094 // 0..3 = ++, +-, -+, -- 

// resolutions
#define kTV1ResolutionVGA					0x8			
#define kTV1ResolutionNTSC					0xF					
#define kTV1ResolutionPAL					0x10			
#define kTV1ResolutionSVGA					0x12		
#define kTV1ResolutionXGAp5994				0x1B					
#define kTV1ResolutionXGAp60				0x1C					
#define kTV1ResolutionXGAp75				0x1D 					
#define kTV1Resolution720p2398				0x27				
#define kTV1Resolution720p24				0x28			
#define kTV1Resolution720p25				0x29			
#define kTV1Resolution720p2997				0x2A				
#define kTV1Resolution720p30				0x2B			
#define kTV1Resolution720p50				0x2C			
#define kTV1Resolution720p5994				0x2D			
#define kTV1Resolution720p60				0x2E			
#define kTV1ResolutionWXGA5by3p60			0x30 						
#define kTV1ResolutionWXGA5by3p75			0x31 						
#define kTV1ResolutionWXGA16by10p60			0x34 						
#define kTV1ResolutionWXGA16by10p75			0x35 		
#define kTV1ResolutionSGAp60				0x3A					
#define kTV1ResolutionSGAp75				0x3B		
#define kTV1ResolutionWSXGAp60				0x40		
#define kTV1ResolutionUXGAp60				0x47					
#define kTV1ResolutionUXGAp75				0x4A					
#define kTV1ResolutionUXGAp85				0x4B		
#define kTV1Resolution1080p2398				0x5B				
#define kTV1Resolution1080p24				0x5C				
#define kTV1Resolution1080p25				0x5D				
#define kTV1Resolution1080p2997				0x5E				
#define kTV1Resolution1080p30				0x5F				
#define kTV1Resolution1080p50				0x60				
#define kTV1Resolution1080p5996				0x61		
#define kTV1Resolution1080p60				0x62				
#define kTV1Resolution1080p75				0x66		
#define kTV1ResolutionWUXGAp60				0x69						
#define kTV1ResolutionWUXGAp75				0x6C						
#define kTV1ResolutionWUXGAp85				0x6D

#define kTV1ResolutionTripleHeadVGAp60		0x70
#define kTV1ResolutionDualHeadSVGAp60		0x71

// keying

#define kTV1FunctionAdjustKeyerEnable				0x127	// Values: 0 = Off, 1 = On
#define kTV1FunctionAdjustKeyerMinY					0xAF	// Values: 0 - 255		
#define kTV1FunctionAdjustKeyerMinU					0xB0	// Values: 0 - 255
#define kTV1FunctionAdjustKeyerMinV					0xB1	// Values: 0 - 255
#define kTV1FunctionAdjustKeyerMaxY					0xB2	// Values: 0 - 255
#define kTV1FunctionAdjustKeyerMaxU					0xB3	// Values: 0 - 255
#define kTV1FunctionAdjustKeyerMaxV					0xB4	// Values: 0 - 255
#define kTV1FunctionAdjustKeyerSoftnessY			0x121	// Values: 0 - 255
#define kTV1FunctionAdjustKeyerSoftnessU			0x123	// Values: 0 - 255
#define kTV1FunctionAdjustKeyerSoftnessV			0x125	// Values: 0 - 255
#define kTV1FunctionAdjustKeyerInvertY				0x122	// Values: 0 = Off, 1 = On
#define kTV1FunctionAdjustKeyerInvertU				0x124	// Values: 0 = Off, 1 = On
#define kTV1FunctionAdjustKeyerInvertV				0x126	// Values: 0 = Off, 1 = On (Typo in documentation, noted as 0x156) 
#define kTV1FunctionAdjustKeyerSwap					0x144	// Values: 0 = Off, 1 = On


class TVOne
{
  public:
    TVOne();
    boolean command(uint8_t channel, uint8_t window, int32_t func, int32_t payload);
    void setCustomResolutions();
    void setHDCPOff();
  private:
    // Tx and Wait LED pins to go here
    void set1920x480(int resStoreNumber);
    void set1600x600(int resStoreNumber);
};

#endif

